/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.task.field;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.util.EnumSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.model.AgroURI;
import nl.cloudfarming.client.model.ItemIdType;
import nl.cloudfarming.client.task.TaskManager;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.util.Exceptions;
import org.openide.util.io.NbObjectInputStream;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 * @author Maciek Dulkiewicz
 */
@ServiceProvider(service = TaskManager.class)
public class FieldTaskManager extends TaskManager<FieldTask> {

    public static final String FIELD_TASK_DIR_NAME = "FieldTasks";
    private static final Logger LOGGER = Logger.getLogger(FieldTaskManager.class.getCanonicalName());

    public void addFieldTask(FieldTask fieldTask) {
        save(fieldTask);
        getPropertyChangeSupport().firePropertyChange(PROP_TASK_ADDED, null, fieldTask);
    }

    @Override
    public FileObject getStorageFolder(Project project) {
        FileObject tasksDir = getTaskDirectory(project);
        FileObject fieldTaskDir = tasksDir.getFileObject(FIELD_TASK_DIR_NAME);
        if (fieldTaskDir == null) {
            try {
                fieldTaskDir = tasksDir.createFolder(FIELD_TASK_DIR_NAME);
            } catch (IOException ex) {
                LOGGER.log(Level.WARNING, "Unable to create FieldTask directory {0}", ex.getMessage());
            }
        }
        return fieldTaskDir;
    }

    @Override
    public Set getSupportedTypes() {
        return EnumSet.of(ItemIdType.FTK);
    }

    public FieldTask loadTask(FileObject fileObject) {
        LOGGER.log(Level.FINEST, "loading FieldTask object from file {0}", fileObject.getName());
        assert FieldTask.EXT.equals(fileObject.getExt()) : "Can only load FieldTasks";
        Object object = null;

        // use of NbObjectInputStream is needed because FieldTask has members from a different module
        // noramel ObjectInputStream uses the module classloader and is not able to see those classes
        try (ObjectInputStream ois = new NbObjectInputStream(fileObject.getInputStream())) {
            object = ois.readObject();
        } catch (ClassNotFoundException | IOException ex) {
            Exceptions.printStackTrace(ex);
            LOGGER.log(Level.SEVERE, "Could not load file {0} with the exception {1}", new Object[]{fileObject, ex});
            throw new RuntimeException(ex);
        }
        assert object instanceof FieldTask : "FileObject did not contain the right type (FieldTask) but a " + object.getClass();
        FieldTask fieldTask = (FieldTask) object;

        return getCachedInstanceFor(fieldTask);
    }

    @Override
    protected FieldTask load(AgroURI uri, Project project) {
        if (uri.getItemIdType().equals(ItemIdType.FTK)) {
            FileObject taskFile = getStorageFolder(project).getFileObject(uri.getItemIdNumber(), FieldTask.EXT);
            if (taskFile != null) {
                return loadTask(taskFile);
            }
        }
        LOGGER.log(Level.WARNING, "Passed AgroURI is not of type FieldTask {0}", uri);
        return null;
    }

    @Override
    public void save(FieldTask task) {
        Project project = TaskManager.findProject(task.getURI());
        save(task, project);
    }

    private void save(FieldTask task, Project project) {
        FileObject storageFolder = getStorageFolder(project);
        FileObject taskFileObject = storageFolder.getFileObject(task.getItemIdNumber(), FieldTask.EXT);

        try {
            if (taskFileObject == null) {
                String taskFileName = task.getItemIdNumber() + "." + FieldTask.EXT;
                try (OutputStream os = storageFolder.createAndOpen(taskFileName)) {
                    save(task, os);
                }
            } else {
                save(task, taskFileObject);
            }
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "Could not create FieldTask file {0} with the exception {1}", new Object[]{task.getItemIdNumber(), ex});
        }

    }

    private void save(FieldTask task, OutputStream outputStream) {
        try (ObjectOutputStream oos = new ObjectOutputStream(outputStream)) {
            oos.writeObject(task);
            addToCache(task);
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "Could not serialize FieldTask to file {0} with the exception {1}", new Object[]{task.getItemIdNumber(), ex});
        }
    }

    private void save(FieldTask task, FileObject targetFile) {
        LOGGER.log(Level.FINEST, "saving field object to file {0}", targetFile.getName());
        assert FieldTask.EXT.equals(targetFile.getExt()) : "Can only save Fields";
        try (ObjectOutputStream oos = new ObjectOutputStream(targetFile.getOutputStream())) {
            oos.writeObject(task);
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "Could not save file {0} with the exception {1}", new Object[]{targetFile, ex});
        }
    }

    private FileObject getTargetFile(String itemIdNumber, Project project) throws IOException {
        FileObject targetFile = getStorageFolder(project).getFileObject(itemIdNumber, FieldTask.EXT);
        if (targetFile == null) {
            targetFile = getStorageFolder(project).createData(itemIdNumber, FieldTask.EXT);
        }

        return targetFile;
    }

    private FileObject getTargetFile(FieldTask fieldTask) throws IOException {
        Project project = TaskManager.findProject(fieldTask.getURI());
        return getTargetFile(fieldTask.getItemIdNumber(), project);
    }

    @Override
    public void remove(FieldTask entity) {
        removeFromCache(entity);
        try {
            getTargetFile(entity).delete();
        } catch (IOException e) {
            LOGGER.log(Level.WARNING, "Could not remove FieldTask {0}. {1}",
                    new Object[]{entity, e.getMessage()
                    });
        }
    }
}
